package ch.unige.solidify.controller;

import static org.springframework.hateoas.server.mvc.WebMvcLinkBuilder.linkTo;

import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.hateoas.RepresentationModel;
import org.springframework.hateoas.server.mvc.WebMvcLinkBuilder;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;

import ch.unige.solidify.SolidifyConstants;
import ch.unige.solidify.repository.SolidifyCompositeRepositoryNoSql;
import ch.unige.solidify.repository.SolidifyRepositoryNoSql;
import ch.unige.solidify.rest.Collection;
import ch.unige.solidify.rest.ResourceNoSql;
import ch.unige.solidify.rest.Tool;

public abstract class AssociationNoSqlReadOnlyController<T extends ResourceNoSql<T>, V extends ResourceNoSql<V>>
        extends SolidifyController {
  protected SolidifyCompositeRepositoryNoSql<T> itemRepository;

  protected SolidifyRepositoryNoSql<V> subResourceService;

  public AssociationNoSqlReadOnlyController(SolidifyCompositeRepositoryNoSql<T> itemRepository,
          SolidifyRepositoryNoSql<V> subResourceService) {
    this.itemRepository = itemRepository;
    this.subResourceService = subResourceService;
  }

  @GetMapping(SolidifyConstants.URL_ID)
  public HttpEntity<V> get(@PathVariable String parentid, @PathVariable String id) {
    this.checkIfResourceExists(parentid);
    final V item = this.itemRepository.findByParentIdAndChildId(parentid, this.getMainResourceClass(),
            this.getSubResourceClass(), id);
    if (item != null) {
      this.addLinks(parentid, item);
      return new ResponseEntity<>(item, HttpStatus.OK);
    } else {
      return new ResponseEntity<>(HttpStatus.NOT_FOUND);
    }
  }

  @GetMapping
  public HttpEntity<Collection<V>> list(@PathVariable String parentid, @ModelAttribute V filterItem,
          Pageable pageable) {
    final T parent = this.checkIfResourceExists(parentid);
    if (parent == null) {
      return new ResponseEntity<>(HttpStatus.NOT_FOUND);
    } else {
      final Page<V> list = this.itemRepository.findByParentId(parentid, this.getMainResourceClass(),
              this.getSubResourceClass(), pageable);

      for (final V v : list) {
        this.addLinks(parentid, v);
      }

      final Collection<V> collection = this.processPageAndLinks(list, parentid, pageable);
      return new ResponseEntity<>(collection, HttpStatus.OK);
    }
  }

  protected void addLinks(String parentid, V v) {
    v.removeLinks();
    v.addLinks(WebMvcLinkBuilder.linkTo(WebMvcLinkBuilder.methodOn(this.getClass())
            .list(parentid, (ResourceNoSql) null, (Pageable) null)));
  }

  protected <W extends RepresentationModel<W>> void addOthersLinks(String parentid, W w) {
  }

  protected T checkIfResourceExists(String id) {
    return this.itemRepository.findOne(this.getIndex(), id);
  }

  protected abstract String getIndex();

  protected abstract Class<T> getMainResourceClass();

  protected abstract Class<V> getSubResourceClass();

  protected Collection<V> processPageAndLinks(Page<V> list, String parentid, Pageable pageable) {
    final Collection<V> collection = new Collection<>(list, pageable);
    collection.add(WebMvcLinkBuilder.linkTo(this.getClass(), parentid).withSelfRel());
    collection.add(Tool.parentLink(
            WebMvcLinkBuilder.linkTo(this.getClass(), parentid).toUriComponentsBuilder())
            .withRel("parent"));
    this.addSortLinks(linkTo(this.getClass(), parentid, pageable), collection);
    this.addPageLinks(WebMvcLinkBuilder.linkTo(this.getClass(), parentid), collection,
            pageable);
    this.addOthersLinks(parentid, collection);
    return collection;
  }

}
