package ch.unige.solidify.rest;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.hateoas.RepresentationModel;

import com.fasterxml.jackson.annotation.JsonInclude;

import ch.unige.solidify.util.CollectionTool;

public class Collection<T> extends RepresentationModel<Collection<T>> {
  private final Iterable<T> _data;
  private final CollectionPage _pageable;
  private final List<FacetResult> _facets = new ArrayList<>();

  public Collection() {
    this._data = Collections.emptyList();
    this._pageable = new CollectionPage();
  }

  public Collection(List<T> list) {
    this._data = list;
    this._pageable = new CollectionPage(list.size());
  }

  public Collection(List<T> list, CollectionPage page) {
    this._data = list;
    this._pageable = page;
  }

  public Collection(List<T> list, CollectionPage page, List<FacetResult> facets) {
    this._data = list;
    this._pageable = page;
    if (facets != null) {
      this._facets.addAll(facets);
    }
  }

  public Collection(Page<T> page, Pageable pageable) {
    this._data = page.getContent();
    this._pageable = new CollectionPage(page, pageable);
    if (page instanceof FacetPage) {
      this._facets.addAll(((FacetPage<T>) page).getFacetResults());
    }
  }

  public Collection(String jsonString, Class<T> itemClass) {
    this._data = CollectionTool.getList(jsonString, itemClass);
    this._pageable = CollectionTool.getPage(jsonString);
    List<FacetResult> facets = CollectionTool.getFacetResults(jsonString);
    if (facets != null) {
      this._facets.addAll(facets);
    }
  }

  public Iterable<T> get_data() {
    return this._data;
  }

  public CollectionPage get_page() {
    return this._pageable;
  }

  @JsonInclude(JsonInclude.Include.NON_EMPTY)
  public List<FacetResult> get_facets() {
    return this._facets;
  }
}
